# Mash MinHash Sketches

More information and an explanation of the Mash
sketch format - not just how to load, but what's
contained in it.

This will start out as mostly just pointers to
relevant oparts of the mash source code,
and lists of things you need.


## Quick Start: Create a Mash MinHash Sketch (JSON Format)

Mash can output MinHash sketches in two formats:

1. capnp format. This format is the default used by Mash. This format uses the
   capnp serialization library to serialize the MinHash sketches and store them
   in a file. **We do not deal with this format** 

2. JSON format. This format is more universal and is easily dealt with
   using any programming language. The only way to generate a JSON file
   with a Mash signature is to create a capnp file first, then convert
   the capnp file to a JSON file.

We cover the process of creating a Mash MinHash in JSON format below.

We will use the sample _E. coli_ genomes provided in the Mash repository
to generate Mash MinHash sketches, and dump them out in JSON format
(which we can load into gopher-smash).

This assumes you are starting from a fresh Ubuntu 18.04 Linux machine.
Start by downloading the mash Linux binary:

```
cd ~/bin/
curl -O https://github.com/marbl/Mash/releases/download/v2.1/mash-Linux64-v2.1.tar
tar xf mash-Linux64-v2.1.tar
cd mash-Linux64-v2.1/
export PATH="${PWD}:${PATH}"
```

Anytime in this shell session you can type `mash` from the command line and
it will call the `mash` binary at `~/mash-Linux64-v2.1/mash`.

Next, clone a copy of the mash repo, which contains 3 sample genomes:

```
cd ~/
git clone https://github.com/marbl/Mash
cd Mash/data/
```

There are three bacterial genomes in .fa format, which we will use to make mash files and mash dump files.

Here are some mash sketch commands:

```
# make a mash sketch in capnp format in file genome1.fna.msh
mash sketch ~/Mash/data/genome1.fna

# make a mash sketch with a specified k-mer length in capnp format in file genome1.fna.msh
mash sketch -k 51 ~/Mash/data/genome1.fna

# make a mash sketch and specify a custom output file name
mash sketch ~/Mash/data/genome2.fna genome_two.msh
```

These `.msh` files are in Capnp format and use a binary serialization
format that is **not** universal, which is why we do not deal with it.

More useful are the MinHash sketches in universal JSON format.
This requires a Capnp file as an input, so you need to run the
`mash sketch` step first (to generate the Capnp `.msh` file),
then run `mash info -d` to dump it to JSON format (dumps to
stdinput by default):

```
mash info -d ~/Mash/data/genome1.fna.msh > genome1.fna.msh.json
```

Voila! We have now exported a Mash MinHash signature to
a JSON file.


## Quick Summary of MinHash

MinHash algorithm:

- list of N hashes
- as you see new values, you keep the minimum hashes you have seen
- k minimum: way of estimating the density and distribution of the values


## Anatomy of a Mash MinHash

Opening the JSON file, we can see the basic components of a MinHash.

Starting with the information at the top, this information
is common to every MinHash sketch that is contained in the file
(potentially a large number):

* kmer size - size of kmers used in the sketch
* alphabet - alphabet for kmer strings
* preserveCase - whether to preserve case of kmer strings
* hashBits - bits used for hash (should always be 64)
* hashSeed - seed for the hash function (42 by default in Mash)
* canonical - whether this is canonical (?)
* hashType - hash function used (should always be `MurmurHash3_x64_128`)
* sketchSize - number of hashes
* sketches (list) - list of each sketch

In the sketches entry, the list is a list of nested JSON.
Here is what it looks like:

* name - name of genome or set of reads
* length - actual (estimated, if using reads) length of genome
* comment - supplementary information about this genome/sequence

Finally, the heart of the MinHash data structure:

* hashes (list) - list of each k-mer hash that appears in this sequence
* maximum - the maximum value in the MinHash structure's hashes list

The hashes list is a flat list of integers.


